<?php
// $Id: ffmpeg.inc,v 1.3.2.6 2008/04/02 12:51:54 jbrown Exp $

// Copyright 2007 Jonathan Brown

define("OP_VIDEO_DEFAULT_FFMPEG_BINARY", 'ffmpeg');


function _op_video_ffmpeg_exec($params, &$output, &$result, $mode = 0) {

	$ffmpeg = variable_get('op_video_ffmpeg_binary', OP_VIDEO_DEFAULT_FFMPEG_BINARY);
	$extra_params = variable_get('op_video_ffmpeg_extra_params', '');

	$command = array($ffmpeg, $params, $extra_params, '2>&1');

	_op_video_exec($command, $output, $result, $mode);
	_op_video_reset_cron();
}


function _op_video_ffmpeg_version_info_fieldset($output) {
	array_shift($output);

	foreach($output as $i => $line) {
		$output[$i] = str_replace('  ', '&nbsp;&nbsp;', $line);
	}

	$output = implode("<br />\n", $output);
	
	$fieldset = array(
		'#type' => 'fieldset',
		'#title' => t('Version info'),
		'#collapsible' => TRUE,
		'#collapsed' => TRUE
	);

	$fieldset[] = array(
		'#value' => '<code>' . $output . '</code>'
	);
	
	return $fieldset;
}


function _op_video_ffmpeg_dot() {

	$url = base_path() . 'misc/menu-leaf.png';

	return array(
		'data' => '<img src="' . $url . '" />', 
		'style' => 'text-align: center;'
	);
}


function _op_video_ffmpeg_formats_fieldset(&$output) {
	$dot = _op_video_ffmpeg_dot();

	while(($line = array_shift($output))) {
		if($line == 'File formats:')
			break;
	}
	
	if(!$line)
		return;
	
	while(($line = array_shift($output)) != '') {

    $rows[] = array(
    	trim(substr($line, 4, 15)),
    	trim(substr($line, 20)),
    	substr($line, 1, 1) == 'D' ? $dot : '',
    	substr($line, 2, 1) == 'E' ? $dot : '',
    );
	}

  $header = array(
  	t('Name'), 
  	t('Description'),
  	array(
			'data' => t('Decode'), 
			'style' => 'text-align: center;'
		),
  	array(
			'data' => t('Encode'), 
			'style' => 'text-align: center;'
		),
  );

	$fieldset = array(
		'#type' => 'fieldset',
		'#title' => t('File formats'),
		'#collapsible' => TRUE,
		'#collapsed' => TRUE
	);

	$fieldset[] = array(
		'#value' => theme('table', $header, $rows)
	);
	
	$note  = 'Note, the names of decoders and encoders do not always match, so there may be cases where the above table ';
	$note .= 'shows decode only or encode only entries even though both decoding and encoding are supported, ';
	$note .= 'e.g. mov / mp4 / 3gp.';

	$fieldset[] = array(
		'#value' => '<p>' . t($note) . '</p>'
	);
	
	return $fieldset;
}


function _op_video_ffmpeg_codecs_fieldset(&$output) {
	$dot = _op_video_ffmpeg_dot();

	$audio[] = array(array(
		'data' => '<b>' . t('Audio codecs') . '</b>', 
		'colspan' => '4'
	));

	$video[] = array(array(
		'data' => '<b>' . t('Video codecs') . '</b>', 
		'colspan' => '4'
	));

	while(($line = array_shift($output))) {
		if($line == 'Codecs:')
			break;
	}

	if(!$line)
		return;

	while(($line = array_shift($output)) != '') {
	
		$codec = array(
			'',
			substr($line, 8),
			substr($line, 1, 1) == 'D' ? $dot : '',
			substr($line, 2, 1) == 'E' ? $dot : '',
		);
		
		switch(substr($line, 3, 1)) {
		
			case 'A':
				$audio[] = $codec;
				break;
				
			case 'V':
				$video[] = $codec;
				break;
		}
	}
	
  $header = array(
		'',
  	t('Name'), 
  	array(
			'data' => t('Decode'), 
			'style' => 'text-align: center;'
		),
  	array(
			'data' => t('Encode'), 
			'style' => 'text-align: center;'
		),
  );

	$fieldset = array(
		'#type' => 'fieldset',
		'#title' => t('Codecs'),
		'#collapsible' => TRUE,
		'#collapsed' => TRUE
	);

	$fieldset[] = array(
		'#value' => theme('table', $header, array_merge($audio, $video))
	);

	$note  = 'Note, the names of decoders and encoders do not always match, so there may be cases where the above table ';
	$note .= 'shows decode only or encode only entries even though both decoding and encoding are supported, ';
	$note .= 'e.g. h264 / libx264.';

	$fieldset[] = array(
		'#value' => '<p>' . t($note) . '</p>'
	);

	return $fieldset;
}


function _op_video_ffmpeg_settings_fieldset($active) {

	$fieldset = array(
		'#type' => 'fieldset',
		'#title' => t('Local FFmpeg installation'),
		'#collapsible' => TRUE,
		'#collapsed' => !$active
	);

	$fieldset['op_video_ffmpeg_binary'] = array(
		'#type' => 'textfield',
		'#title' => t('FFmpeg binary'),
		'#description' => t('If the binary is not in the default path, you should enter the absolute path to it.'),
		'#default_value' => variable_get('op_video_ffmpeg_binary', OP_VIDEO_DEFAULT_FFMPEG_BINARY),
	);
	
	$fieldset['op_video_ffmpeg_extra_params'] = array(
		'#type' => 'textfield',
		'#title' => t('Extra parameters (advanced)'),
		'#description' => t('Any extra <a href="http://ffmpeg.mplayerhq.hu/ffmpeg-doc.html">parameters</a> you wish to have appended onto the FFmpeg command.'),
		'#default_value' => variable_get('op_video_ffmpeg_extra_params', ''),
	);
	
	$output = array();
	_op_video_ffmpeg_exec('-version', $output, $result);
	
	if($result == 0) {
		$fieldset[] = _op_video_ffmpeg_version_info_fieldset($output);

		$output = array();
		_op_video_ffmpeg_exec('-formats', $output, $result);

		$fieldset[] = _op_video_ffmpeg_formats_fieldset($output);
		$fieldset[] = _op_video_ffmpeg_codecs_fieldset($output);
	}
	else {
		$fieldset[] = array(
			'#value' => '<p><b>' . t('FFmpeg binary not found.') . '</b></p>'
		);
	}
	
	return $fieldset;
}


function _op_video_ffmpeg_get_info($source_file) {
	
	$params = '-i ' . escapeshellarg($source_file['filepath']);
	$output = array();
	_op_video_ffmpeg_exec($params, $output, $result);
	array_shift($output);
	$output = implode("\n", $output);
	
	if(preg_match('/Duration: *(..):(..):(..\..), /', $output, $matches))
		$info['duration'] = ($matches[1] * 60 + $matches[2]) * 60 + $matches[3];
	else
		return NULL;

	if(preg_match('/Audio: (.*)/', $output, $matches)) {
		$parts = explode(', ', $matches[1]);
		
		$info['audio_codec'] = strtok(array_shift($parts), ' ');
		$info['audio_sample_rate'] = strtok(array_shift($parts), ' ');
		$info['audio_channels'] = array_shift($parts);
		
		switch($info['audio_channels']) {
			
			case 'mono':
				$info['audio_channels'] = 1;
				break;

			case 'stereo':
				$info['audio_channels'] = 2;
				break;
		}
		
		$info['audio_bitrate'] = strtok(array_shift($parts), ' ');
	}
	else {
		$info['audio_channels'] = 0;
	}

	if(preg_match('/Video: (.*)/', $output, $matches)) {
		$parts = explode(', ', $matches[1]);
		
		$info['video_codec'] = array_shift($parts);
		array_shift($parts);
		$dims = explode('x', array_shift($parts));
		$info['width'] = $dims[0];
		$info['height'] = strtok($dims[1], ' ');
		
		if(count($parts) > 1)
			$info['video_bitrate'] = strtok(array_shift($parts), ' ');
			
		$info['frame_rate'] = strtok(array_shift($parts), ' ');
	}
	
	$info['video_bitrate'] = $source_file['filesize'] * 8 / 1000 / $info['duration'] - $info['audio_bitrate'];

	$info['source_file'] = $source_file;
	
	return $info;
}


function _op_video_ffmpeg_create_png($id, $info, $i, &$output) {
	
	$params[] = '-y -i ' . escapeshellarg($info['source_file']['filepath']);

	$out_file = _op_video_temp_path() . '/' . $id . '_raw_image_' . $i . '.png';

	if($i == 2)
		$params[] = '-ss ' . $info['duration'] / 3;

	$params[] = '-vcodec png';
	$params[] = '-vframes 1';
	$params[] = '-f rawvideo';
	$params[] = '-an';
	$params[] = $out_file;

	_op_video_ffmpeg_exec($params, $output, $result, OP_VIDEO_EXEC_MODE_TIME | OP_VIDEO_EXEC_MODE_ECHO);

	if($result == 0) {
		file_move($out_file, _op_video_transcoded_path(), FILE_EXISTS_REPLACE);
		return _op_video_insert_filepath($out_file);
	}
	else {
		file_delete($out_file);
		return FALSE;
	}
}


function _op_video_ffmpeg_create_flv($id, $info, $dirty_params, &$output) {

	$params[] = '-y -i ' . escapeshellarg($info['source_file']['filepath']);

	if($dirty_params['max_duration'] != 0 && 
		$info['duration'] > $dirty_params['max_duration'])
	{
		$params[] = '-t ' . $dirty_params['max_duration'];
	}

	if(in_array($info['video_codec'], array('flv', 'vp6f', 'h264')) &&
		$info['video_bitrate'] <= $dirty_params['video_bitrate'] &&
		$info['width'] <= $dirty_params['width'] &&
		$info['frame_rate'] <= $dirty_params['frame_rate'])
	{
		$ext = $info['video_codec'] == 'h264' ? 'mp4' : 'flv';
		$params[] = '-vcodec copy';
		$passthrough = 1;
	}
	else {
		$ext = 'flv';
//		$params[] = '-vcodec flv';
		$params[] = '-b ' . $dirty_params['video_bitrate'] . 'k';

		if($info['frame_rate'] > $dirty_params['frame_rate']) {
			$frame_rate = $dirty_params['frame_rate'];
			$params[] = '-r ' . $frame_rate;
		}
		else {
			$frame_rate = $info['frame_rate'];
		}
		
		$params[] = '-g ' . (int)round($frame_rate * $dirty_params['key_interval']);

		if($info['width'] > $dirty_params['width']) {

			$width = ((int)($dirty_params['width'] / 2)) * 2;
			$height = (int)round(($dirty_params['width'] * $info['height']) / $info['width']);
			$height = ((int)($height / 2)) * 2;
			
			$params[] = '-s ' . $width . 'x' . $height;
		}
	}
	
	$channels = min($info['audio_channels'], $dirty_params['audio_channels']);

	if($channels == 0) {
		$params[] = '-an';
	}
	else if((($info['audio_codec'] == 'mp3' &&
		in_array($info['audio_sample_rate'], array(11025, 22050, 44100))) ||
		($info['audio_codec'] == 'mpeg4aac' && $ext == 'mp4')) &&
		$info['audio_bitrate'] <= $dirty_params['audio_bitrate'] &&
		$info['audio_channels'] <= $dirty_params['audio_channels'] &&
		$info['audio_sample_rate'] <= $dirty_params['audio_sample_rate'])
	{
		$params[] = '-acodec copy';
	}
	else {
//		$params[] = '-acodec ' . ($ext == 'mp4' ? 'libfaac' : 'libmp3lame');
		$params[] = '-ab ' . $dirty_params['audio_bitrate'] . 'k';
		$params[] = '-ac ' . $channels;
		$params[] = '-ar ' . $dirty_params['audio_sample_rate'];
	}
	
	$out_file = _op_video_temp_path() . '/' . $id . '.' . $ext;
	$params[] = $out_file;

	if($passthrough) {
		_op_video_ffmpeg_exec($params, $output, $result, OP_VIDEO_EXEC_MODE_TIME | OP_VIDEO_EXEC_MODE_ECHO);
	}
	else {
		$log_file = _op_video_temp_path() . '/2pass.' . $id . '.log';

		_op_video_ffmpeg_exec(array($params, '-pass 1 -passlogfile ' . $log_file), 
			$output, $result, OP_VIDEO_EXEC_MODE_TIME | OP_VIDEO_EXEC_MODE_ECHO);

		if($result == 0) {
			_op_video_ffmpeg_exec(array($params, '-pass 2 -passlogfile ' . $log_file), 
				$output, $result, OP_VIDEO_EXEC_MODE_TIME | OP_VIDEO_EXEC_MODE_ECHO);

			if($result != 0) {	// dual pass failed - try single pass
				_op_video_ffmpeg_exec($params, $output, $result, OP_VIDEO_EXEC_MODE_TIME | OP_VIDEO_EXEC_MODE_ECHO);
			}
		}

		file_delete($log_file);
	}
	
	if($result == 0) {
		file_move($out_file, _op_video_transcoded_path(), FILE_EXISTS_REPLACE);
		return _op_video_insert_filepath($out_file);
	}
	else {
		file_delete($out_file);
		return FALSE;
	}
}


function _op_video_transcode_ffmpeg($video) {

	$info = _op_video_ffmpeg_get_info($video['source_file']);
	
	if($info) {
		$flv_file_id = _op_video_ffmpeg_create_flv($video['video_id'], $info, $video['dirty_params'], $output);

		if($flv_file_id) {
			$start_image_file_id = _op_video_ffmpeg_create_png($video['video_id'], $info, 1, $output);
			$third_image_file_id = _op_video_ffmpeg_create_png($video['video_id'], $info, 2, $output);

			$current_params_id = _op_video_transcoding_params_insert($video['dirty_params']);

			db_query("
				UPDATE {op_videos}
				SET duration = %d,
					flv_file_id = %d,
					start_image_file_id = %d,
					third_image_file_id = %d,
					current_params_id = %d,
					status = 'idle',
					error = 'none'
				WHERE video_id = %d
			",
				(int)ceil($info['duration']),
				$flv_file_id,
				$start_image_file_id,
				$third_image_file_id,
				$current_params_id,
				$video['video_id']
			);

			_op_video_transcoding_completed($video);
			return;
		}
	}

	db_query("
		UPDATE {op_videos}
		SET status = 'idle',
			error = 'transcoding failed'
		WHERE video_id = %d
	",
		$video['video_id']
	);
}

