<?php
// $Id: field.inc,v 1.1.2.17 2008/04/02 12:51:54 jbrown Exp $

// Copyright 2007-2008 Jonathan Brown

define("OP_VIDEO_DEFAULT_ALLOW_REPLACE", 0);
define("OP_VIDEO_DEFAULT_ALLOW_EMBED", 0);


/*
			FIELD
*/


function op_video_field_info() {

  return array(
    'op_video' => array('label' => "Video")
  );
}


function _op_video_get_params_id($field) {
	$params_ids = variable_get('op_video_field_params_ids', array());
	return $params_ids[$field['field_name']];
}


function _op_video_field_load_params($field) {
	$params_ids = variable_get('op_video_field_params_ids', array());

	if(!$params_ids[$field['field_name']]) {
		$params_ids[$field['field_name']] = _op_video_transcoding_params_insert();
		variable_set('op_video_field_params_ids', $params_ids);
	}
	
	return _op_video_transcoding_params_load($params_ids[$field['field_name']]);
}


function _op_video_field_retranscode_all($field) {

	$table = 'content_type_' . $field['type_name'];
	$video_id_field = $field['field_name'] . '_video_id';

/*
	db_query("
		UPDATE {{$table}}, {op_videos}, {op_video_files}
		SET {op_videos}.secret = '%s',
			{op_videos}.status = 'dirty', 
			{op_videos}.upload_failed = 0,
			{op_videos}.uploaded = 0
		WHERE {{$table}}.$video_id_field = {op_videos}.video_id AND
			{op_videos}.source_file_id = {op_video_files}.file_id AND
			{op_video_files}.deleted = 0
	",
		md5(user_password())
	);
*/

	$result = db_query("
		SELECT {op_videos}.video_id
		FROM {op_videos}
		INNER JOIN {{$table}}
			ON {op_videos}.video_id = {{$table}}.$video_id_field
		INNER JOIN {op_video_files}
			ON {op_videos}.source_file_id = {op_video_files}.file_id
		WHERE {op_video_files}.deleted = 0
	");

	while($video = db_fetch_array($result)) {
		_op_video_retranscode_video($video['video_id']);
	}

	$msg = t('Retranscoding all field %label videos.', array('%label' => $field['label']));
	drupal_set_message($msg);
	watchdog('op_video', $msg);
}


function _op_video_field_store_params($field) {
	$params_id = _op_video_get_params_id($field);
	_op_video_transcoding_params_update($params_id, $field);
	
	if($field['retranscode_all']) {
		_op_video_field_retranscode_all($field);
	}
}


function op_video_field_settings($op, $field) {

  switch ($op) {

    case 'form':
			$description = t("Allow the video the be embedded on other websites.") . '<br />';
			$description .= t("Users with <em>administer video</em> permission can always do this.");

			$form['allow_embed'] = array(
				'#type' => 'checkbox',
				'#title' => 'Allow embedding',
				'#default_value' => isset($field['allow_embed']) ? $field['allow_embed'] : OP_VIDEO_DEFAULT_ALLOW_EMBED,
				'#description' => $description
			);
			
     	$form['preview_image_settings'] = _op_video_preview_image_settings_fieldset($field);
     	$form['player_settings'] = _op_video_player_settings_fieldset($field);
    	
    	$params = _op_video_field_load_params($field);
    	$form['transcoding_params'] = _op_video_transcoding_params_fieldset($params);
    	
			return $form;


    case 'validate':
    
    	_op_video_preview_image_settings_validate($field);
    	_op_video_player_settings_validate($field);
    	_op_video_transcoding_params_validate($field);
    	
    	if(!form_get_errors()) {		// $form data is not passed to 'save' op below
	    	_op_video_field_store_params($field);
    	}
			return;


    case 'save':
    
			_op_video_purge_image_cache();
			$preview_image_fields = _op_video_player_settings_fields();
			$player_fields = _op_video_preview_image_settings_fields();
			return array_merge(array('allow_embed'), $preview_image_fields, $player_fields);

    case 'database columns':

      return array(
				'video_id' => array('type' => 'int', 'unsigned' => TRUE, 'not null' => TRUE)
      );
      
      
		case 'tables':
		
			$tables = content_views_field_tables($field);
			
			$table_name = array_shift(array_keys($tables));
			$field_name = array_shift(array_keys($tables[$table_name]['fields']));
			
			unset($tables[$table_name]['fields']);
			
			$tables[$field['field_name'] . '_op_videos'] = array(
				'name' => 'op_videos',
				'join' => array(
					'left' => array(
						'table' => $table_name,
						'field' => $field_name
					),
					'right' => array(
						'field' => 'video_id'
					)
				),
		    'fields' => _op_video_views_op_videos_fields($field['widget']['label']),
//		    'filters' => _op_video_views_op_videos_filters($field['widget']['label'])
		  );

			$tables[$field['field_name'] . '_op_video_files'] = 
				_op_video_views_op_video_files_table($field['field_name'], $field['widget']['label']);
			
			return $tables;
  }
}


function op_video_field($op, &$node, $field, &$node_field, $teaser, $page) {

	if($node_field[0]['custom_image_file_id']) {
		$node_field[0]['preview_image']['preview_image'] = 'custom';
	}

	switch ($op) {

		case 'insert':
			$params_id = _op_video_get_params_id($field);

			$node_field[0]['video_id'] = 
				_op_video_create_video($node_field[0]['source_file_id'], $params_id, 
					$node_field[0]['preview_image']['preview_image'], $node_field[0]['custom_image_file_id'],
					$node->nid, $node_field[0]['flv_file_id'], $node_field[0]['duration']);
			
			break;
			
		case 'update':
			$video = _op_video_load($node_field[0]['video_id']);
			
			if($node_field[0]['upload']['delete']) {			// video deleted
				_op_video_delete($video);
				$node_field[0]['video_id'] = 0;
			}
			else if($node_field[0]['source_file_id']) {		// video replaced

				$custom_image_file_id = $node_field[0]['custom_image_file_id'] ? 
					$node_field[0]['custom_image_file_id'] : $video['custom_image_file_id'];
			
				$params_id = _op_video_get_params_id($field);
				$node_field[0]['video_id'] = 
					_op_video_create_video($node_field[0]['source_file_id'], $params_id, 
						$node_field[0]['preview_image']['preview_image'], $custom_image_file_id, $node->nid);
				
				if($custom_image_file_id == $video['custom_image_file_id']) {
					$video['custom_image_file_id'] = NULL;
					unset($video['custom_image_file']);
				}
				
				_op_video_delete($video);
			}
			else {																				// video kept
			
				if($node_field[0]['preview_image']['upload']['delete']) {
					_op_video_update_custom_image($video, 0);
					
					if($node_field[0]['preview_image']['preview_image'] == 'custom')
						$node_field[0]['preview_image']['preview_image'] = $field['preview_image_image'];
				}
				else if($node_field[0]['custom_image_file_id']) {
					_op_video_update_custom_image($video, $node_field[0]['custom_image_file_id']);
				}
			
				if($node_field[0]['preview_image']['preview_image'] && 
					($node_field[0]['preview_image']['preview_image'] != $video['preview_image'])) {
					_op_video_set_preview_image($node_field[0]['video_id'], $node_field[0]['preview_image']['preview_image']);
				}
				
				if($node_field[0]['retranscode']) {
					_op_video_retranscode_video($node_field[0]['video_id']);
					drupal_set_message('The video will be retranscoded.');
				}
			}

			break;

		case 'delete':
			_op_video_delete($node_field[0]['video_id']);
			break;
	}
}


/*
			WIDGET
*/


function op_video_widget_info() {

  return array(
    'upload' => array(
      'label' => t('Upload'),
      'field types' => array('op_video'),
    )
  );
}


function op_video_widget_settings($op, $widget) {

  switch ($op) {

    case 'form':
    
			$description = t("Allow the video the be replaced / deleted after the node has been created.") . "<br />\n";
			$description .= t("Users with <em>administer video</em> permission can always do this.");

			$form['allow_replace'] = array(
				'#type' => 'checkbox',
				'#title' => 'Allow replacement',
				'#default_value' => isset($widget['allow_replace']) ? $widget['allow_replace'] : OP_VIDEO_DEFAULT_ALLOW_REPLACE,
				'#description' => $description
			);
			
      return $form;

    case 'save':
    	return array('allow_replace');

    case 'callbacks':
      return array(
        'view' => CONTENT_CALLBACK_CUSTOM,
      );
  }
}


function _op_video_widget_upload_form_item($text, $field, $existing, $filepath, $required = FALSE) {

	if(!count($_POST)) {

		// this is a fresh form - delete old preview video if it exists

		$file = file_save_upload($field['field_name'], file_directory_temp());

		if($file)
			file_delete($file->filepath);
	}

	if($existing) {
	
		if(!$required) {
		  $item['delete'] = array(
		    '#type' => 'checkbox',
		    '#title' => t('Delete this') . ' ' . t($text),
		  );
		}
		 
		$item[] = array(
			'#value' => '<div style="clear: both;"></div>'
		);
	}
	
	$file = file_check_upload($field['field_name']);

	if($file) {
		$info  = 'Filename: ' . $file->filename . '<br />';
		$info .= 'Filesize: ' . _op_video_format_bytesize($file->filesize, 2);
		$item['uploaded'] = array(
			'#type' => 'fieldset',
			'#title' => t('Uploaded') . ' ' . t($text)
		);
		$item['uploaded'][] = array(
			'#value' => $info
		);
    $item['uploaded']['ignore'] = array(
      '#type' => 'checkbox',
      '#title' => t('Ignore')
    );
	}
	
	$item[$field['field_name']] = array(
		'#type' => 'file', 
		'#name' => 'files[' . $field['field_name'] . ']',
		'#description' => 'Max size: ' . _op_video_upload_max_size()
	);

	if(!$existing && !$file)
		$item[$field['field_name']]['#title'] = t('Upload') . ' ' . t($text);
	else
		$item[$field['field_name']]['#title'] = t('Upload new') . ' ' . t($text);

	if(user_access('administer video')) {
		$item['filepath'] = array(
			'#type' => 'textfield',
			'#title' => 'Filepath',
			'#autocomplete_path' => 'op_video/path_autocomplete',
			'#default_value' => $filepath
		);
	}
	
	return $item;
}


function _op_video_widget_preview_image($field, $node_field) {

	$item = array(
		'#type' => 'fieldset',
		'#title' => t('Preview image')
	);
	
	if($node_field[0]['video_id']) {
		$video = _op_video_load($node_field[0]['video_id']);
	}
	
	if(_op_video_does_file_exist($video['start_image_file']))
		$options['start'] = _op_video_render_preview_image($video, 80, 60, NULL, 'start') . ' - start of video';
	else
		$options['start'] = 'Start of video';
	
	if(_op_video_does_file_exist($video['third_image_file']))
		$options['third'] = _op_video_render_preview_image($video, 80, 60, NULL, 'third') . ' - @ 1/3 of video duration';
	else
		$options['third'] = '@ 1/3 of video duration';

	$existing = _op_video_does_file_exist($video['custom_image_file']);

	if($existing)
		$options['custom'] = _op_video_render_preview_image($video, 80, 60, NULL, 'custom') . ' - custom image';

	$item['preview_image'] = array(
		'#type' => 'radios',
		'#default_value' => isset($node_field[0]['preview_image']) ? $node_field[0]['preview_image'] : 
			(isset($field['preview_image_image']) ? $field['preview_image_image'] : OP_VIDEO_DEFAULT_PREVIEW_IMAGE_IMAGE),
		'#options' => $options
	);

	$field['field_name'] .= '_image';
	$item['upload'] = _op_video_widget_upload_form_item('custom image', $field, $existing, 
		_op_video_does_file_really_exist($video['custom_image_file']) ? $video['custom_image_file']['filepath'] : '');

	return $item;
}


function _op_video_widget_upload_validate($name, $preview) {

	if($preview) {
		$file = file_save_upload($name);
	}
	else {
		$file = file_check_upload($name);
	}

	if($file) {
		if($_SESSION['file_uploads'][$name]->filepath != $file->filepath)
			file_delete($_SESSION['file_uploads'][$name]->filepath);
	}
	
	return $file;
}


function op_video_widget($op, &$node, $field, &$node_field) {

  switch ($op) {

		case 'prepare form values':
			$video = _op_video_load($node_field[0]['video_id']);
			$node_field[0]['preview_image'] = $video['preview_image'];
			break;


    case 'form':

  		_op_video_create_dirs();
  		
			$form[$field['field_name']]['#weight'] = $field['widget']['weight'];
			$form[$field['field_name']]['#tree'] = TRUE;
  		$form[$field['field_name']][0] = array(
  			'#type' => 'fieldset',
				'#title' => t($field['widget']['label'])
  		);
			
			$existing = $node_field[0]['video_id'];
			
			if($existing) {
			
				$form[$field['field_name']][0]['video_id'] = array(
					'#type' => 'value',
					'#value' => $node_field[0]['video_id']
				);

				$video = _op_video_load($node_field[0]['video_id']);
				$filepath = _op_video_does_file_really_exist($video['source_file']) ? $video['source_file']['filepath'] : '';
				
				$info = _op_video_render_preview_image($video, 240, 180, array('style="float: left; margin-right: 10px;"'));
			
				$info .= 'Filename: ' . $video['source_file']['filename'] . '<br />';
				$info .= 'Filesize: ' . _op_video_format_bytesize($video['source_file']['filesize'], 2) . '<br />';
				
				if($video['duration'])
					$info .= 'Duration: ' . format_interval($video['duration']) . '<br />';
				
				$form[$field['field_name']][0][] = array(
					'#value' => $info
				);

				if(user_access('administer video') && 
					_op_video_does_file_exist($video['source_file'])) {
					
					$form[$field['field_name']][0]['retranscode'] = array(
						'#type' => 'checkbox', 
						'#title' => t('Retranscode this video')
					);
				}
			}
			
			if(!$existing || $field['widget']['allow_replace'] || user_access('administer video')) {
				$form[$field['field_name']][0]['upload'] = 
					_op_video_widget_upload_form_item('video', $field, $existing, $filepath, $field['required']);
			}
			else {
				$form[$field['field_name']][0][] = array(
					'#value' => '<div style="clear: both;"></div>'
				);
			}			

			if(isset($field['preview_image_user_selectable']) ? $field['preview_image_user_selectable'] :
				OP_VIDEO_DEFAULT_PREVIEW_IMAGE_USER_SELECTABLE) // || user_access('administer video'))
			{
	      $form[$field['field_name']][0]['preview_image'] = _op_video_widget_preview_image($field, $node_field);
			}

			$form['#attributes']['enctype'] = 'multipart/form-data';
			
      return $form;


		case 'validate':
		
			$file = _op_video_widget_upload_validate($field['field_name'], $node->op == t('Preview'));

			if(!$file && $node_field[0]['upload']['filepath']) {
				if(!is_file($node_field[0]['upload']['filepath'])) {
					form_set_error($field['field_name'] . '][0][upload][filepath', t('File not found.'));
				}
			}

			if(!$node_field[0]['video_id'] && $field['required'] &&
				!$node_field[0]['upload']['filepath'] && !($file && !$node_field[0]['upload']['uploaded']['ignore']))
			{
				form_set_error($field['field_name'] . '][0][upload]['. $field['field_name'], t('You need to upload a video.'));
			}

			$file = _op_video_widget_upload_validate($field['field_name'] . '_image', $node->op == t('Preview'));
			
			if(!$file && $node_field[0]['preview_image']['upload']['filepath']) {
				if(!is_file($node_field[0]['preview_image']['upload']['filepath'])) {
					form_set_error($field['field_name'] . '][0][preview_image][upload][filepath', t('File not found.'));
				}
			}
			
			break;


		case 'submit':
		
			if($node_field[0]['video_id'])
				$video = _op_video_load($node_field[0]['video_id']);

			if($node_field[0]['upload']['delete'] || 
				$node_field[0]['upload']['uploaded']['ignore'])
			{
				$file = file_save_upload($field['field_name'], file_directory_temp());

				if($file)
					file_delete($file->filepath);
					
				unset($file);
			}
			else {
				$file = file_check_upload($field['field_name']);
			}

			if($file) {
				$node_field[0]['source_file_id'] = _op_video_save_upload($field['field_name'], 'original');
			}
			else {
				if(_op_video_canonical_filepath($node_field[0]['upload']['filepath']) != 
					$video['source_file']['filepath'])
				{
					$node_field[0]['source_file_id'] = 
						_op_video_insert_filepath($node_field[0]['upload']['filepath'], TRUE);
				}
			}
			
			if($video || $node_field[0]['source_file_id']) {
			
				if($node_field[0]['preview_image']['upload']['delete'] || 
					$node_field[0]['preview_image']['upload']['uploaded']['ignore'])
				{
					$file = file_save_upload($field['field_name'] . '_image', file_directory_temp());

					if($file)
						file_delete($file->filepath);
					
					unset($file);
				}
				else {
					$file = file_check_upload($field['field_name'] . '_image');
				}
				
				if($file) {
					$node_field[0]['custom_image_file_id'] = _op_video_save_upload($field['field_name'] . '_image', 'original');
				}
				else {
					if(_op_video_canonical_filepath($node_field[0]['preview_image']['upload']['filepath']) != 
						$video['custom_image_file']['filepath'])
					{
						$node_field[0]['custom_image_file_id'] = 
							_op_video_insert_filepath($node_field[0]['preview_image']['upload']['filepath'], TRUE);
					}
				}
			}

			break;
	}
}


/*
			FORMATTERS
*/


function op_video_field_formatter_info() {

	return array(
		'default' => array(
			'label' => 'Inline player',
			'field types' => array('op_video'),
		),
		'preview-image-link' => array(
			'label' => 'Preview image - link to node',
			'field types' => array('op_video'),
		),
		'preview-image-layer' => array(
			'label' => 'Preview image - play in layer',
			'field types' => array('op_video'),
		)
	);
}


function op_video_field_formatter($field, $item, $formatter, $node) {

	_op_video_set_render_settings($field);
	
	$video = _op_video_load($item['video_id']);
	
	return _op_video_render_video($video, $formatter, $node->nid, TRUE);
}


function _op_video_get_first_video_field($node) {
	$content_type_info = _content_type_info();
	
	foreach((array)$node as $field => $value) {
		if($content_type_info['fields'][$field]['type'] == 'op_video') {
    	return $field;
    }
	}
}


function _op_video_get_first_video($node) {

	$field = _op_video_get_first_video_field($node);
	
	if($field)
	 	return _op_video_load($node->{$field}[0]['video_id']);
}


function op_video_nodeapi(&$node, $op, $teaser, $page) {

	if($op != 'rss item')
		return;
		
	$video = _op_video_get_first_video($node);

	if($video && !$video['source_file']['deleted']) {

		$rss_items[] = array(
			'key' => 'enclosure',
			'attributes' => array(
				'url' => file_create_url($video['source_file']['filepath']),
				'length' => $video['source_file']['filesize'],
				'type' => $video['source_file']['filemime']
			)
		);
		
    return $rss_items;
	}
}

